@@ -21,12 +21,16 @@
 #ifndef APP_H
 #define APP_H
 
-#include <kxmlguiwindow.h>
+// KDE
+#include <KXmlGuiWindow>
 
+// Qt
 class QLabel;
 
+// KShisen
 class Board;
 
+
 /**
  * @brief Class holding the application and its functions
  */
@@ -35,7 +39,7 @@ class App : public KXmlGuiWindow
     Q_OBJECT
 
 public:
-    explicit App(QWidget *parent = 0);
+    explicit App(QWidget * parent = nullptr);
 
 private slots:
     void slotEndOfGame();
@@ -80,7 +84,7 @@ private slots:
      * If the game is paused, do not show the board and disable actions like undo
      * and such.
      */
-    void setPauseEnabled(bool enable);
+    void setPauseEnabled(bool enabled);
 
     /** Undoes one move.
      * The Undo action should set the cheat flag, so the user cannot end up in
@@ -128,11 +132,11 @@ private:
     void setCheatModeEnabled(bool enabled);
 
 private:
-    QLabel *m_gameTipLabel; ///< Status bar area for game tips
-    QLabel *m_gameTimerLabel; ///< Status bar area for the timer
-    QLabel *m_gameTilesLabel; ///< Status bar area for the tile counter
-    QLabel *m_gameCheatLabel; ///< Status bar area for the cheat mode
-    Board *m_board; ///< Holds the game board
+    QLabel * m_gameTipLabel; ///< Status bar area for game tips
+    QLabel * m_gameTimerLabel; ///< Status bar area for the timer
+    QLabel * m_gameTilesLabel; ///< Status bar area for the tile counter
+    QLabel * m_gameCheatLabel; ///< Status bar area for the cheat mode
+    Board * m_board; ///< Holds the game board
 };
 
 #endif // APP_H
