@@ -24,39 +24,35 @@
 #ifndef BOARD_H
 #define BOARD_H
 
-#include "debug.h"
+// STL
+#include <vector>
 
-#include <KgSound>
-
-#include <kgameclock.h>
-#include <kmahjonggtileset.h>
-#include <kmahjonggbackground.h>
-
-#include <krandomsequence.h>
-
-#include "kshisen_debug.h"
+// Qt
 #include <QList>
 #include <QSize>
 #include <QWidget>
 
-// used in board.cpp and app.cpp, thus defined here
-static int sizeX[6] = {14, 16, 18, 24, 26, 30};
-static int sizeY[6] = { 6,  9,  8, 12, 14, 16};
+// KDE
+#include <KRandomSequence>
 
-/**
- * @brief Struct holding a position on the board (x,y)
- */
-struct Position {
-    Position() : x(0), y(0) { }
-    Position(int _x, int _y) : x(_x), y(_y) { }
-    int x; ///< x position
-    int y; ///< y position
-};
+// KDEGames
+#include <KGameClock>
+#include <KgSound>
+
+// KMahjongg
+#include <kmahjonggbackground.h>
+#include <kmahjonggtileset.h>
+
+// KShisen
+#include "debug.h"
+
+
+using TilePos = QPoint;
 
 /**
  * A list of positions (at least 2) makes a Path
  */
-typedef QList<Position> Path;
+using Path = QList<TilePos>;
 
 /**
  * @brief Class holding a possible move and its functions
@@ -69,24 +65,33 @@ typedef QList<Position> Path;
 class PossibleMove
 {
 public:
-    explicit PossibleMove(Path &path) :
-        m_path(path), m_hasSlide(false) { }
-    PossibleMove(Path &path, Path &slide) :
-        m_path(path), m_hasSlide(true), m_slide(slide) { }
+    explicit PossibleMove(Path & path)
+        : m_path(path)
+        , m_hasSlide(false)
+        , m_slide()
+    {
+    }
+    PossibleMove(Path & path, Path & slide)
+        : m_path(path)
+        , m_hasSlide(true)
+        , m_slide(slide)
+    {
+    }
 
-    bool isInPath(int x, int y) const;
+    bool isInPath(TilePos const & tilePos) const;
 
-    void Debug() const {
+    void Debug() const
+    {
         qCDebug(KSHISEN_LOG) << "PossibleMove";
 
-        for (auto iter = m_path.constBegin(); iter != m_path.constEnd(); ++iter) {
-            qCDebug(KSHISEN_LOG) << "    Path:" << iter->x << "," << iter->y;
+        foreach (auto iter, m_path) {
+            qCDebug(KSHISEN_LOG) << "    Path:" << iter.x() << "," << iter.y();
         }
 
         if (m_hasSlide) {
             qCDebug(KSHISEN_LOG) << "   hasSlide";
-            for (auto iter = m_slide.constBegin(); iter != m_slide.constEnd(); ++iter) {
-                qCDebug(KSHISEN_LOG) << "    Slide:" << iter->x << "," << iter->y;
+            foreach (auto iter, m_slide) {
+                qCDebug(KSHISEN_LOG) << "    Slide:" << iter.x() << "," << iter.y();
             }
         }
     }
@@ -99,7 +104,7 @@ public:
 /**
  * A list of possible moves the player has to choose between
  */
-typedef QList<PossibleMove> PossibleMoves;
+using PossibleMoves = QList<PossibleMove>;
 
 
 /**
@@ -110,12 +115,48 @@ typedef QList<PossibleMove> PossibleMoves;
 class Move
 {
 public:
-    Move(int x1, int y1, int x2, int y2, int tile) :
-        m_x1(x1), m_y1(y1), m_x2(x2), m_y2(y2), m_tile1(tile), m_tile2(tile), m_hasSlide(false), m_slideX1(-1), m_slideY1(-1), m_slideX2(-1), m_slideY2(-1) { }
-    Move(int x1, int y1, int x2, int y2, int tile1, int tile2) :
-        m_x1(x1), m_y1(y1), m_x2(x2), m_y2(y2), m_tile1(tile1), m_tile2(tile2), m_hasSlide(false), m_slideX1(-1), m_slideY1(-1), m_slideX2(-1), m_slideY2(-1) { }
-    Move(int x1, int y1, int x2, int y2, int tile1, int tile2, int slideX1, int slideY1, int slideX2, int slideY2) :
-        m_x1(x1), m_y1(y1), m_x2(x2), m_y2(y2), m_tile1(tile1), m_tile2(tile2), m_hasSlide(true), m_slideX1(slideX1), m_slideY1(slideY1), m_slideX2(slideX2), m_slideY2(slideY2) { }
+    Move(TilePos const & tilePos1, TilePos const & tilePos2, int tile)
+        : m_x1(tilePos1.x())
+        , m_y1(tilePos1.y())
+        , m_x2(tilePos2.x())
+        , m_y2(tilePos2.y())
+        , m_tile1(tile)
+        , m_tile2(tile)
+        , m_hasSlide(false)
+        , m_slideX1(-1)
+        , m_slideY1(-1)
+        , m_slideX2(-1)
+        , m_slideY2(-1)
+    {
+    }
+    Move(TilePos const & tilePos1, TilePos const & tilePos2, int tile1, int tile2)
+        : m_x1(tilePos1.x())
+        , m_y1(tilePos1.y())
+        , m_x2(tilePos2.x())
+        , m_y2(tilePos2.y())
+        , m_tile1(tile1)
+        , m_tile2(tile2)
+        , m_hasSlide(false)
+        , m_slideX1(-1)
+        , m_slideY1(-1)
+        , m_slideX2(-1)
+        , m_slideY2(-1)
+    {
+    }
+    Move(TilePos const & tilePos1, TilePos const & tilePos2, int tile1, int tile2, int slideX1, int slideY1, int slideX2, int slideY2)
+        : m_x1(tilePos1.x())
+        , m_y1(tilePos1.y())
+        , m_x2(tilePos2.x())
+        , m_y2(tilePos2.y())
+        , m_tile1(tile1)
+        , m_tile2(tile2)
+        , m_hasSlide(true)
+        , m_slideX1(slideX1)
+        , m_slideY1(slideY1)
+        , m_slideX2(slideX2)
+        , m_slideY2(slideY2)
+    {
+    }
 
     int m_x1, m_y1, m_x2, m_y2; ///< coordinates of the two tiles that matched
     int m_tile1; ///< type of tile at first set of coordinates
@@ -136,17 +177,17 @@ class Board : public QWidget
     Q_OBJECT
 
 public:
-    Board(QWidget *parent = 0);
-    ~Board();
+    explicit Board(QWidget * parent = nullptr);
 
-    static const int nTiles = 42;
+    /// Number of different kinds of tiles in the game.
+    static int const nTiles = 42;
 
-    virtual void paintEvent(QPaintEvent *e);
-    virtual void mousePressEvent(QMouseEvent *e);
-    virtual void resizeEvent(QResizeEvent *e);
+    virtual void paintEvent(QPaintEvent * e);
+    virtual void mousePressEvent(QMouseEvent * e);
+    virtual void resizeEvent(QResizeEvent * e);
 
     void setDelay(int);
-    int  delay() const;
+    int delay() const;
 
     /// Returns if undo step is available
     bool canUndo() const;
@@ -161,13 +202,13 @@ public:
     void resizeBoard();
 
     void showHint();
-    bool hint_I(PossibleMoves &possibleMoves) const;
+    bool hint_I(PossibleMoves & possibleMoves) const;
 
 #ifdef DEBUGGING
     void makeHintMove();
     void finish();
     void dumpBoard() const;
-    void dumpBoard(const int *) const;
+    void dumpBoard(const std::vector<int> & board) const;
 #endif
 
     /// Returns the number of tiles left on the board
@@ -179,14 +220,18 @@ public:
     bool solvable(bool noRestore = false); // const?
 
     bool solvableFlag() const;
-    void setSolvableFlag(bool b);
+    void setSolvableFlag(bool enabled);
     bool gravityFlag() const;
-    void setGravityFlag(bool b);
-    void setChineseStyleFlag(bool b);
-    void setTilesCanSlideFlag(bool b);
+    void setGravityFlag(bool enabled);
+    void setChineseStyleFlag(bool enabled);
+    void setTilesCanSlideFlag(bool enabled);
 
+    /// Returns possible number of tiles in X direction
     int xTiles() const;
+    /// Returns possible number of tiles in Y direction
     int yTiles() const;
+    /// Returns overall possible number of tiles in current game board size
+    int tiles() const;
 
     /// Resets the game timer
     void resetTimer();
@@ -225,6 +270,7 @@ signals:
     void markMatched(); // unused?
     void newGameStarted();
     void changed();
+    void tileCountChanged();
     void endOfGame();
     void resized();
     void invalidMove();
@@ -252,9 +298,9 @@ public slots:
     /// Loads the game settings
     void loadSettings();
     /// Loads the given tileset
-    bool loadTileset(const QString &);
+    bool loadTileset(QString const & pathToTileset);
     /// Loads the given background
-    bool loadBackground(const QString &);
+    bool loadBackground(QString const & pathToBackground);
 
 private slots:
     void undrawConnection();
@@ -281,10 +327,10 @@ private: // functions
      */
     int lineWidth() const;
 
-    void setField(int x, int y, int value);
-    int field(int x, int y) const;
-    void updateField(int, int);
-    void showInfoRect(QPainter &, const QString &message);
+    void setField(TilePos const & tilePos, int value);
+    int field(TilePos const & tilePos) const;
+    void updateField(TilePos const & tilePos);
+    void showInfoRect(QPainter &, const QString & message);
     void drawTiles(QPainter &, QPaintEvent *);
     void clearHighlight();
 
@@ -294,51 +340,47 @@ private: // functions
     bool tilesMatch(int tile1, int tile2) const;
 
     /** Checks if a path between two tiles can be made with a single line.
-     * @param x1 x coordinate of the first tile
-     * @param y1 y coordinate of the first tile
-     * @param x2 x coordinate of the second tile
-     * @param y2 y coordinate of the second tile
+     * @param tilePos1 coordinates of the first tile
+     * @param tilePos2 coordinates of the second tile
      */
-    bool canMakePath(int x1, int y1, int x2, int y2) const;
+    bool canMakePath(TilePos const & tilePos1, TilePos const & tilePos2) const;
 
-    /** Checks if the tile at (x1,y1) can be slid to (x2,y2).
-     * @param x1 x coordinate of the slide's initial position
-     * @param y1 y coordinate of the slide's initial position
-     * @param x2 x coordinate of the slide's final position
-     * @param y2 y coordinate of the slide's final position
+    /** Checks if the tile at \p tilePos1 can be slid to \p tilePos2.
+     * @param tilePos1 coordinates of the slide's initial position
+     * @param tilePos2 coordinates of the slide's final position
      * @param path The movement of the last tile slided will be stored in the path
      */
-    bool canSlideTiles(int x1, int y1, int x2, int y2, Path &path) const;
+    bool canSlideTiles(TilePos const & tilePos1, TilePos const & tilePos2, Path & path) const;
 
     /** Checks if a path between two tiles can be made with 2 or 3 lines.
-    * @param x1 x coordinate of the first tile
-    * @param y1 y coordinate of the first tile
-    * @param x2 x coordinate of the second tile
-    * @param y2 y coordinate of the second tile
+    * @param tilePos1 coordinate of the first tile
+    * @param tilePos2 coordinate of the second tile
     * @param possibleMoves All the possible moves are stored here
     * @return The number of paths found
     */
-    int findPath(int x1, int y1, int x2, int y2, PossibleMoves &possibleMoves) const;
+    int findPath(TilePos const & tilePos1, TilePos const & tilePos2, PossibleMoves & possibleMoves) const;
 
     /** Find a path of 1 or 2 segments between tiles.
-     * @param x1 x coordinate of the first tile
-     * @param y1 y coordinate of the first tile
-     * @param x2 x coordinate of the second tile
-     * @param y2 y coordinate of the second tile
+     * @param tilePos1 coordinates of the first tile
+     * @param tilePos2 coordinates of the second tile
      * @param possibleMoves All the possible moves are stored here
      * @return The number of paths found
      */
-    int findSimplePath(int x1, int y1, int x2, int y2, PossibleMoves &possibleMoves) const;
-    void performMove(PossibleMove &possibleMoves);
-    void performSlide(int x, int y, Path& s);
-    void reverseSlide(int x, int y, int slideX1, int slideY1, int slideX2, int slideY2);
-    bool isTileHighlighted(int x, int y) const;
+    int findSimplePath(TilePos const & tilePos1, TilePos const & tilePos2, PossibleMoves & possibleMoves) const;
+    void performMove(PossibleMove & possibleMoves);
+    void performSlide(TilePos const & tilePos, Path & slide);
+    void reverseSlide(TilePos const & tilePos, int slideX1, int slideY1, int slideX2, int slideY2);
+    bool isTileHighlighted(TilePos const & tilePos) const;
     void drawConnection();
     void drawPossibleMoves(bool b);
-    QPoint midCoord(int x, int y) const;
+    /** Calculated the middle coordinates of the given tile position.
+     * @param tilePos Tile position
+     * @return The middle coordinates of the tile at \p tilePos
+     */
+    QPoint midCoord(TilePos const & tilePos) const;
     void unmarkTile();
-    void marked(int x, int y);
-    void madeMove(int x1, int y1, int x2, int y2, Path slide = Path());
+    void marked(TilePos const & tilePos);
+    void madeMove(TilePos const & tilePos1, TilePos const & tilePos2, Path slide = Path());
 
     /** Checks all columns and populate the affected columns in m_gravCols.
      * @param update FIXME: What is it for?
@@ -353,21 +395,25 @@ private:
 
     KRandomSequence m_random;
 
-    QList<Move*> m_undo; ///< Undo history
-    QList<Move*> m_redo; ///< Redo history
+    QList<Move *> m_undo; ///< Undo history
+    QList<Move *> m_redo; ///< Redo history
 
     int m_markX;
     int m_markY;
     Path m_connection;
     PossibleMoves m_possibleMoves;
-    int *m_field; ///< Matrix pointer holding the game board grid
+    std::vector<int> m_field; ///< Matrix holding the game board grid
     int m_xTiles;
     int m_yTiles;
     int m_delay;
     int m_level;
     int m_shuffle;
 
-    enum class GameState { Normal, Paused, Stuck, Over }; ///< Whether game is paused, has no more matching tiles or is over
+    // The game can be in one of the following states.
+    enum class GameState { Normal,
+                           Paused,
+                           Stuck,
+                           Over };
     GameState m_gameState;
     bool m_cheat; ///< Whether the cheat mode is set
 
@@ -382,8 +428,8 @@ private:
     bool m_paintConnection;
     bool m_paintPossibleMoves;
     bool m_paintInProgress;
-    QPair<int, int> m_tileRemove1;
-    QPair<int, int> m_tileRemove2;
+    TilePos m_tileRemove1;
+    TilePos m_tileRemove2;
     KgSound m_soundPick; ///< Sound object to play when tile is selected
     KgSound m_soundFall; ///< Sound object to play when tiles fall down in gravity mode
 };
